@@ -2,7 +2,7 @@ import asyncio
 import collections
 import unittest
 from unittest import mock
-from aiohttp.multidict import MultiDict
+from aiohttp.multidict import CIMultiDict
 from aiohttp.web import Request
 from aiohttp.protocol import RawRequestMessage, HttpVersion11
 
@@ -17,6 +17,7 @@ class TestHTTPExceptions(unittest.TestCase):
 
         self.payload = mock.Mock()
         self.transport = mock.Mock()
+        self.reader = mock.Mock()
         self.writer = mock.Mock()
         self.writer.drain.return_value = ()
         self.buf = b''
@@ -29,12 +30,12 @@ class TestHTTPExceptions(unittest.TestCase):
     def append(self, data):
         self.buf += data
 
-    def make_request(self, method='GET', path='/', headers=MultiDict()):
+    def make_request(self, method='GET', path='/', headers=CIMultiDict()):
         self.app = mock.Mock()
         message = RawRequestMessage(method, path, HttpVersion11, headers,
                                     False, False)
         req = Request(self.app, message, self.payload,
-                      self.transport, self.writer, 15)
+                      self.transport, self.reader, self.writer)
         return req
 
     def test_all_http_exceptions_exported(self):
