@@ -1,18 +1,71 @@
 """Tests for http/websocket.py"""
 
+import asyncio
 import base64
 import hashlib
 import os
+import random
 import struct
 import unittest
 import unittest.mock
 
 import aiohttp
 from aiohttp import websocket, multidict, protocol, errors
+from aiohttp.websocket import Message
 
 
 class WebsocketParserTests(unittest.TestCase):
 
+    def setUp(self):
+        self.loop = asyncio.new_event_loop()
+        asyncio.set_event_loop(None)
+
+        self.buf = aiohttp.ParserBuffer()
+        self.out = aiohttp.DataQueue(loop=self.loop)
+
+    def tearDown(self):
+        self.loop.close()
+
+    def build_frame(self, message, opcode, use_mask=False, noheader=False):
+        """Send a frame over the websocket with message as its payload."""
+        msg_length = len(message)
+        if use_mask:  # pragma: no cover
+            mask_bit = 0x80
+        else:
+            mask_bit = 0
+
+        if msg_length < 126:
+            header = websocket.PACK_LEN1(
+                0x80 | opcode, msg_length | mask_bit)
+        elif msg_length < (1 << 16):  # pragma: no cover
+            header = websocket.PACK_LEN2(
+                0x80 | opcode, 126 | mask_bit, msg_length)
+        else:
+            header = websocket.PACK_LEN3(
+                0x80 | opcode, 127 | mask_bit, msg_length)
+
+        if use_mask:  # pragma: no cover
+            mask = random.randrange(0, 0xffffffff)
+            mask = mask.to_bytes(4, 'big')
+            message = websocket._websocket_mask(mask, bytearray(message))
+            if noheader:
+                return message
+            else:
+                return header + mask + message
+        else:
+            if noheader:
+                return message
+            else:
+                return header + message
+
+    def build_close_frame(self, code=1000, message=b'', noheader=False):
+        """Close the websocket, sending the specified code and message."""
+        if isinstance(message, str):  # pragma: no cover
+            message = message.encode('utf-8')
+        return self.build_frame(
+            websocket.PACK_CLOSE_CODE(code) + message,
+            opcode=websocket.OPCODE_CLOSE, noheader=noheader)
+
     def test_parse_frame(self):
         buf = aiohttp.ParserBuffer()
         p = websocket.parse_frame(buf)
@@ -121,14 +174,11 @@ class WebsocketParserTests(unittest.TestCase):
             yield
             return (1, websocket.OPCODE_PING, b'data')
         m_parse_frame.side_effect = parse_frame
-        buf = aiohttp.ParserBuffer()
-        p = websocket.parse_message(buf)
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
-        try:
-            p.send(b'')
-        except StopIteration as exc:
-            res = exc.value
-        self.assertEqual(res, (websocket.OPCODE_PING, b'data', ''))
+        p.send(b'')
+        res = self.out._buffer[0]
+        self.assertEqual(res, ((websocket.OPCODE_PING, b'data', ''), 4))
 
     @unittest.mock.patch('aiohttp.websocket.parse_frame')
     def test_pong_frame(self, m_parse_frame):
@@ -136,14 +186,11 @@ class WebsocketParserTests(unittest.TestCase):
             yield
             return (1, websocket.OPCODE_PONG, b'data')
         m_parse_frame.side_effect = parse_frame
-        buf = aiohttp.ParserBuffer()
-        p = websocket.parse_message(buf)
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
-        try:
-            p.send(b'')
-        except StopIteration as exc:
-            res = exc.value
-        self.assertEqual(res, (websocket.OPCODE_PONG, b'data', ''))
+        p.send(b'')
+        res = self.out._buffer[0]
+        self.assertEqual(res, ((websocket.OPCODE_PONG, b'data', ''), 4))
 
     @unittest.mock.patch('aiohttp.websocket.parse_frame')
     def test_close_frame(self, m_parse_frame):
@@ -151,13 +198,11 @@ class WebsocketParserTests(unittest.TestCase):
             yield
             return (1, websocket.OPCODE_CLOSE, b'')
         m_parse_frame.side_effect = parse_frame
-        p = websocket.parse_message(aiohttp.ParserBuffer())
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
-        try:
-            p.send(b'')
-        except StopIteration as exc:
-            res = exc.value
-        self.assertEqual(res, (websocket.OPCODE_CLOSE, 0, ''))
+        p.send(b'')
+        res = self.out._buffer[0]
+        self.assertEqual(res, ((websocket.OPCODE_CLOSE, 0, ''), 0))
 
     @unittest.mock.patch('aiohttp.websocket.parse_frame')
     def test_close_frame_info(self, m_parse_frame):
@@ -165,14 +210,12 @@ class WebsocketParserTests(unittest.TestCase):
             yield
             return (1, websocket.OPCODE_CLOSE, b'0112345')
         m_parse_frame.side_effect = parse_frame
-        buf = aiohttp.ParserBuffer()
-        p = websocket.parse_message(buf)
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
-        try:
-            p.send(b'')
-        except StopIteration as exc:
-            res = exc.value
-        self.assertEqual(res, (websocket.OPCODE_CLOSE, 12337, b'12345'))
+        p.send(b'')
+        res = self.out._buffer[0]
+        self.assertEqual(
+            res, (Message(websocket.OPCODE_CLOSE, 12337, '12345'), 0))
 
     @unittest.mock.patch('aiohttp.websocket.parse_frame')
     def test_close_frame_invalid(self, m_parse_frame):
@@ -180,10 +223,29 @@ class WebsocketParserTests(unittest.TestCase):
             yield
             return (1, websocket.OPCODE_CLOSE, b'1')
         m_parse_frame.side_effect = parse_frame
-        buf = aiohttp.ParserBuffer()
-        p = websocket.parse_message(buf)
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
-        self.assertRaises(websocket.WebSocketError, p.send, b'')
+        with self.assertRaises(websocket.WebSocketError) as ctx:
+            next(p)
+
+        self.assertEqual(ctx.exception.code, websocket.CLOSE_PROTOCOL_ERROR)
+
+    def test_close_frame_invalid_2(self):
+        self.buf.extend(self.build_close_frame(code=1))
+        p = websocket.WebSocketParser(self.out, self.buf)
+        with self.assertRaises(websocket.WebSocketError) as ctx:
+            next(p)
+
+        self.assertEqual(ctx.exception.code, websocket.CLOSE_PROTOCOL_ERROR)
+
+    def test_close_frame_unicode_err(self):
+        self.buf.extend(self.build_close_frame(
+            code=1000, message=b'\xf4\x90\x80\x80'))
+        p = websocket.WebSocketParser(self.out, self.buf)
+        with self.assertRaises(websocket.WebSocketError) as ctx:
+            next(p)
+
+        self.assertEqual(ctx.exception.code, websocket.CLOSE_INVALID_TEXT)
 
     @unittest.mock.patch('aiohttp.websocket.parse_frame')
     def test_unknown_frame(self, m_parse_frame):
@@ -191,25 +253,26 @@ class WebsocketParserTests(unittest.TestCase):
             yield
             return (1, websocket.OPCODE_CONTINUATION, b'')
         m_parse_frame.side_effect = parse_frame
-        buf = aiohttp.ParserBuffer()
-        p = websocket.parse_message(buf)
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
         self.assertRaises(websocket.WebSocketError, p.send, b'')
 
-    @unittest.mock.patch('aiohttp.websocket.parse_frame')
-    def test_simple_text(self, m_parse_frame):
-        def parse_frame(buf):
-            yield
-            return (1, websocket.OPCODE_TEXT, b'text')
-        m_parse_frame.side_effect = parse_frame
-        buf = aiohttp.ParserBuffer()
-        p = websocket.parse_message(buf)
+    def test_simple_text(self):
+        self.buf.extend(self.build_frame(b'text', websocket.OPCODE_TEXT))
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
-        try:
-            p.send(b'')
-        except StopIteration as exc:
-            res = exc.value
-        self.assertEqual(res, (websocket.OPCODE_TEXT, 'text', ''))
+        p.send(b'')
+        res = self.out._buffer[0]
+        self.assertEqual(res, ((websocket.OPCODE_TEXT, 'text', ''), 4))
+
+    def test_simple_text_unicode_err(self):
+        self.buf.extend(
+            self.build_frame(b'\xf4\x90\x80\x80', websocket.OPCODE_TEXT))
+        p = websocket.WebSocketParser(self.out, self.buf)
+        with self.assertRaises(websocket.WebSocketError) as ctx:
+            next(p)
+
+        self.assertEqual(ctx.exception.code, websocket.CLOSE_INVALID_TEXT)
 
     @unittest.mock.patch('aiohttp.websocket.parse_frame')
     def test_simple_binary(self, m_parse_frame):
@@ -217,20 +280,17 @@ class WebsocketParserTests(unittest.TestCase):
             yield
             return (1, websocket.OPCODE_BINARY, b'binary')
         m_parse_frame.side_effect = parse_frame
-        buf = aiohttp.ParserBuffer()
-        p = websocket.parse_message(buf)
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
-        try:
-            p.send(b'')
-        except StopIteration as exc:
-            res = exc.value
-        self.assertEqual(res, (websocket.OPCODE_BINARY, b'binary', ''))
+        p.send(b'')
+        res = self.out._buffer[0]
+        self.assertEqual(res, ((websocket.OPCODE_BINARY, b'binary', ''), 6))
 
     @unittest.mock.patch('aiohttp.websocket.parse_frame')
     def test_continuation(self, m_parse_frame):
         cur = 0
 
-        def parse_frame(buf):
+        def parse_frame(buf, cont=False):
             nonlocal cur
             yield
             if cur == 0:
@@ -240,21 +300,44 @@ class WebsocketParserTests(unittest.TestCase):
                 return (1, websocket.OPCODE_CONTINUATION, b'line2')
 
         m_parse_frame.side_effect = parse_frame
-        buf = aiohttp.ParserBuffer()
-        p = websocket.parse_message(buf)
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
         p.send(b'')
-        try:
-            p.send(b'')
-        except StopIteration as exc:
-            res = exc.value
-        self.assertEqual(res, (websocket.OPCODE_TEXT, 'line1line2', ''))
+        p.send(b'')
+        res = self.out._buffer[0]
+        self.assertEqual(
+            res, (Message(websocket.OPCODE_TEXT, 'line1line2', ''), 10))
+
+    @unittest.mock.patch('aiohttp.websocket.parse_frame')
+    def test_continuation_with_ping(self, m_parse_frame):
+        frames = [
+            (0, websocket.OPCODE_TEXT, b'line1'),
+            (0, websocket.OPCODE_PING, b''),
+            (1, websocket.OPCODE_CONTINUATION, b'line2'),
+        ]
+
+        def parse_frame(buf, cont=False):
+            yield
+            return frames.pop(0)
+
+        m_parse_frame.side_effect = parse_frame
+        p = websocket.WebSocketParser(self.out, self.buf)
+        next(p)
+        p.send(b'')
+        p.send(b'')
+        p.send(b'')
+        res = self.out._buffer[0]
+        self.assertEqual(
+            res, (Message(websocket.OPCODE_PING, b'', ''), 0))
+        res = self.out._buffer[1]
+        self.assertEqual(
+            res, (Message(websocket.OPCODE_TEXT, 'line1line2', ''), 10))
 
     @unittest.mock.patch('aiohttp.websocket.parse_frame')
     def test_continuation_err(self, m_parse_frame):
         cur = 0
 
-        def parse_frame(buf):
+        def parse_frame(buf, cont=False):
             nonlocal cur
             yield
             if cur == 0:
@@ -264,53 +347,130 @@ class WebsocketParserTests(unittest.TestCase):
                 return (1, websocket.OPCODE_TEXT, b'line2')
 
         m_parse_frame.side_effect = parse_frame
-        buf = aiohttp.ParserBuffer()
-        p = websocket.parse_message(buf)
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
         p.send(b'')
         self.assertRaises(websocket.WebSocketError, p.send, b'')
 
-    @unittest.mock.patch('aiohttp.websocket.parse_message')
-    def test_parser(self, m_parse_message):
-        cur = 0
+    @unittest.mock.patch('aiohttp.websocket.parse_frame')
+    def test_continuation_with_close(self, m_parse_frame):
+        frames = [
+            (0, websocket.OPCODE_TEXT, b'line1'),
+            (0, websocket.OPCODE_CLOSE,
+             self.build_close_frame(1002, b'test', noheader=True)),
+            (1, websocket.OPCODE_CONTINUATION, b'line2'),
+        ]
 
-        def parse_message(buf):
-            nonlocal cur
+        def parse_frame(buf, cont=False):
             yield
-            if cur == 0:
-                cur = 1
-                return websocket.Message(websocket.OPCODE_TEXT, b'line1', b'')
-            else:
-                return websocket.Message(websocket.OPCODE_CLOSE, b'', b'')
+            return frames.pop(0)
 
-        m_parse_message.side_effect = parse_message
-        out = aiohttp.FlowControlDataQueue(unittest.mock.Mock())
-        buf = aiohttp.ParserBuffer()
-        p = websocket.WebSocketParser(out, buf)
+        m_parse_frame.side_effect = parse_frame
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
         p.send(b'')
-        self.assertRaises(StopIteration, p.send, b'')
-
+        p.send(b'')
+        p.send(b'')
+        res = self.out._buffer[0]
         self.assertEqual(
-            (websocket.OPCODE_TEXT, b'line1', b''), out._buffer[0])
+            res, (Message(websocket.OPCODE_CLOSE, 1002, 'test'), 0))
+        res = self.out._buffer[1]
         self.assertEqual(
-            (websocket.OPCODE_CLOSE, b'', b''), out._buffer[1])
-        self.assertTrue(out._eof)
+            res, (Message(websocket.OPCODE_TEXT, 'line1line2', ''), 10))
 
-    def test_parser_eof(self):
-        out = aiohttp.FlowControlDataQueue(unittest.mock.Mock())
-        buf = aiohttp.ParserBuffer()
-        p = websocket.WebSocketParser(out, buf)
+    @unittest.mock.patch('aiohttp.websocket.parse_frame')
+    def test_continuation_with_close_unicode_err(self, m_parse_frame):
+        frames = [
+            (0, websocket.OPCODE_TEXT, b'line1'),
+            (0, websocket.OPCODE_CLOSE,
+             self.build_close_frame(1000, b'\xf4\x90\x80\x80', noheader=True)),
+            (1, websocket.OPCODE_CONTINUATION, b'line2')]
+
+        def parse_frame(buf, cont=False):
+            yield
+            return frames.pop(0)
+
+        m_parse_frame.side_effect = parse_frame
+        p = websocket.WebSocketParser(self.out, self.buf)
         next(p)
-        self.assertRaises(aiohttp.EofStream, p.throw, aiohttp.EofStream)
-        self.assertEqual([], list(out._buffer))
+        p.send(b'')
+        with self.assertRaises(websocket.WebSocketError) as ctx:
+            p.send(b'')
+
+        self.assertEqual(ctx.exception.code, websocket.CLOSE_INVALID_TEXT)
+
+    @unittest.mock.patch('aiohttp.websocket.parse_frame')
+    def test_continuation_with_close_bad_code(self, m_parse_frame):
+        frames = [
+            (0, websocket.OPCODE_TEXT, b'line1'),
+            (0, websocket.OPCODE_CLOSE,
+             self.build_close_frame(1, b'test', noheader=True)),
+            (1, websocket.OPCODE_CONTINUATION, b'line2')]
+
+        def parse_frame(buf, cont=False):
+            yield
+            return frames.pop(0)
+
+        m_parse_frame.side_effect = parse_frame
+        p = websocket.WebSocketParser(self.out, self.buf)
+        next(p)
+        p.send(b'')
+        with self.assertRaises(websocket.WebSocketError) as ctx:
+            p.send(b'')
+
+        self.assertEqual(ctx.exception.code, websocket.CLOSE_PROTOCOL_ERROR)
+
+    @unittest.mock.patch('aiohttp.websocket.parse_frame')
+    def test_continuation_with_close_bad_payload(self, m_parse_frame):
+        frames = [
+            (0, websocket.OPCODE_TEXT, b'line1'),
+            (0, websocket.OPCODE_CLOSE, b'1'),
+            (1, websocket.OPCODE_CONTINUATION, b'line2')]
+
+        def parse_frame(buf, cont=False):
+            yield
+            return frames.pop(0)
+
+        m_parse_frame.side_effect = parse_frame
+        p = websocket.WebSocketParser(self.out, self.buf)
+        next(p)
+        p.send(b'')
+        with self.assertRaises(websocket.WebSocketError) as ctx:
+            p.send(b'')
+
+        self.assertEqual(ctx.exception.code, websocket.CLOSE_PROTOCOL_ERROR)
+
+    @unittest.mock.patch('aiohttp.websocket.parse_frame')
+    def test_continuation_with_close_empty(self, m_parse_frame):
+        frames = [
+            (0, websocket.OPCODE_TEXT, b'line1'),
+            (0, websocket.OPCODE_CLOSE, b''),
+            (1, websocket.OPCODE_CONTINUATION, b'line2'),
+        ]
+
+        def parse_frame(buf, cont=False):
+            yield
+            return frames.pop(0)
+
+        m_parse_frame.side_effect = parse_frame
+        p = websocket.WebSocketParser(self.out, self.buf)
+        next(p)
+        p.send(b'')
+        p.send(b'')
+        p.send(b'')
+        res = self.out._buffer[0]
+        self.assertEqual(
+            res, (Message(websocket.OPCODE_CLOSE, 0, ''), 0))
+        res = self.out._buffer[1]
+        self.assertEqual(
+            res, (Message(websocket.OPCODE_TEXT, 'line1line2', ''), 10))
 
 
 class WebsocketWriterTests(unittest.TestCase):
 
     def setUp(self):
         self.transport = unittest.mock.Mock()
-        self.writer = websocket.WebSocketWriter(self.transport)
+        self.writer = websocket.WebSocketWriter(self.transport, use_mask=False)
 
     def test_pong(self):
         self.writer.pong()
@@ -335,9 +495,12 @@ class WebsocketWriterTests(unittest.TestCase):
 
     def test_send_binary_very_long(self):
         self.writer.send(b'b' * 65537, True)
-        self.assertTrue(
-            self.transport.write.call_args[0][0].startswith(
-                b'\x82\x7f\x00\x00\x00\x00\x00\x01\x00\x01b'))
+        self.assertEqual(
+            self.transport.write.call_args_list[0][0][0],
+            b'\x82\x7f\x00\x00\x00\x00\x00\x01\x00\x01')
+        self.assertEqual(
+            self.transport.write.call_args_list[1][0][0],
+            b'b' * 65537)
 
     def test_close(self):
         self.writer.close(1001, 'msg')
@@ -346,6 +509,14 @@ class WebsocketWriterTests(unittest.TestCase):
         self.writer.close(1001, b'msg')
         self.transport.write.assert_called_with(b'\x88\x05\x03\xe9msg')
 
+    def test_send_text_masked(self):
+        writer = websocket.WebSocketWriter(self.transport,
+                                           use_mask=True,
+                                           random=random.Random(123))
+        writer.send(b'text')
+        self.transport.write.assert_called_with(
+            b'\x81\x84\rg\xb3fy\x02\xcb\x12')
+
 
 class WebSocketHandshakeTests(unittest.TestCase):
 
